# vim: expandtab tabstop=4 shiftwidth=4

from pathlib import Path
from typing import Dict

from .exceptions import MinnowPropertiesException

def has_reserved_characters(s: str) -> bool:
    if '=' in s: return True
    if '\n' in s: return True
    return False

def load_properties(path: Path) -> Dict[str, str]:
    with path.open('r') as f:
        lines = f.read().split('\n')

    pairs = [line.split('=') for line in lines]
    valid_pairs = [pair for pair in pairs if len(pair) == 2]
    return {k.strip(): v.strip() for k, v in valid_pairs}

def save_properties(properties: Dict, path:Path) -> None:
    pairs = [(str(k).strip(), str(v).strip()) for k, v in properties.items() if type(v) is not bool]
    pairs += [(str(k).strip(), str(v).strip().lower()) for k, v in properties.items() if type(v) is bool]  # True becomes true

    for k, v in pairs:
        if has_reserved_characters(k):
            raise MinnowPropertiesException('"{}" is not a valid property key: contains reserved characters'.format(k))

        if has_reserved_characters(v):
            raise MinnowPropertiesException('"{}" is not a valid property value: contains reserved characters'.format(k))

    lines = ['{k} = {v}'.format(k=k, v=v) for k, v in pairs]
    data = '\n'.join(lines)

    with path.open('w') as f:
        f.write(data)



# vim: expandtab tabstop=4 shiftwidth=4

from pathlib import Path
from typing import List

from .exceptions import MinnowPathException

class DataMetadataPair:
    def __init__(self, data_path: Path, metadata_path: Path) -> None:
        self.data_path = data_path
        self.metadata_path = metadata_path

def list_pairs_at_path(input_dir: Path, extension: str='.properties') -> List[DataMetadataPair]:
    if not input_dir.exists():
        raise MinnowPathException('Cannot list_pairs_at_path. Path does not exist: {}'.format(input_dir))

    if len(extension) == 0:
        raise MinnowPathException('Cannot list_pairs_at_path with file extension of length zero.')

    if not extension.startswith('.'):
        raise MinnowPathException('Cannot list_pairs_at_path. Extension must start with a dot.')

    metadata_paths = list(input_dir.glob('*'+extension))
    data_paths = [Path(mp.with_suffix('')) for mp in metadata_paths]
    pairs = [DataMetadataPair(data_paths[i], metadata_paths[i]) for i in range(len(data_paths)) if data_paths[i].exists()]
    return pairs



# vim: expandtab tabstop=4 shiftwidth=4

import logging

from pathlib import Path

from .utils import list_pairs_at_path

class Processor:
    def __init__(self, input_path: Path, output_path: Path) -> None:
        self.input_path = input_path
        self.output_path = output_path
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.addHandler(logging.StreamHandler())
        self.logger.setLevel(logging.DEBUG)

    def process(self, metadata_path: Path, data_path: Path, output_path: Path) -> None:
        self.logger.info('Received {mp} and {dp}, with output path {op}'.format(mp=metadata_path, dp=data_path, op=output_path))
        self.logger.error('process() not defined')

    def run(self):
        for pair in list_pairs_at_path(self.input_path):
            if not pair.metadata_path.exists():
                self.logger.error('Metadata path does not exist: {}'.format(pair.metadata_path))
            elif not pair.data_path.exists():
                self.logger.error('Data path does not exist: {}'.format(pair.data_path))
            else:
                self.process(pair.metadata_path, pair.data_path, self.output_path)



# vim: expandtab tabstop=4 shiftwidth=4

from .exceptions import MinnowException, MinnowPathException, MinnowPropertiesException
from .processor import Processor
from .properties import load_properties, save_properties
from .utils import DataMetadataPair, list_pairs_at_path



# vim: expandtab tabstop=4 shiftwidth=4

from setuptools import setup

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), 'r') as f:
    long_description = f.read()

setup(
    name='minnow',
    version='2020.307.0',
    author='Bill Allen',
    author_email='photo.allen@gmail.com',
    description='Utilities for the Minnow file processing framework.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    keywords='minnow file processing properties'.split(),
    url='https://github.com/gershwinlabs/minnow-python',
    packages=['minnow'],
    install_requires=['setuptools'],
    python_requires='>=3.5, <4',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License'
    ]
)
